@@ -7,15 +7,16 @@
 
 using namespace pybindcpp;
 
+double my_cos(double x) { return cos(x); }
+float my_sin(float x) { return sin(x); }
 double fn(long N, double x) { return N * x; }
-
 long add_one(long x) { return x + 1; }
 
-void import(module m) {
-  add_ufunc<double, double>(m, "cos", cos);
-  add_ufunc<float, float>(m, "sin", sin);
+void init(module m) {
+  add_ufunc<double, double>(m, "cos", my_cos);
+  add_ufunc<float, float>(m, "sin", my_sin);
   add_ufunc<double, long, double>(m, "fn", fn);
   add_ufunc<long, long>(m, "add_one", add_one);
 }
 
-PYBINDCPP_INIT(ufunc, import)
+PYBINDCPP_INIT(ufunc, init)
