@@ -1,4 +1,4 @@
-// Copyright (C) 2010-2016 Dzhelil S. Rufat. All Rights Reserved.
+// Copyright (C) 2010-2019 Dzhelil S. Rufat. All Rights Reserved.
 #ifndef PYBINDCPP_CTYP_TYPES_H
 #define PYBINDCPP_CTYP_TYPES_H
 
@@ -17,12 +17,11 @@ template <typename T> static size_t type_id() {
   return id;
 }
 
-template <class T> struct ctype_trait;
+template <class T> struct ctype;
 
 template <class T, class... Args> struct sexpr {
-
   static size_t add(const char *name) {
-    const size_t args[] = {ctype_trait<Args>::add()...};
+    const size_t args[] = {ctype<Args>::add()...};
     const size_t nargs = sizeof...(Args);
     auto tid = type_id<T>();
     return ts->add_type(tid, name, args, nargs);
@@ -30,30 +29,19 @@ template <class T, class... Args> struct sexpr {
 };
 
 template <class T> struct sexpr<T> {
-
   static size_t add(const char *name) {
     auto tid = type_id<T>();
     return ts->add_type(tid, name, nullptr, 0);
   }
 };
 
-template <> struct ctype_trait<void> {
-  static auto add() { return sexpr<void>::add("void"); }
-};
-
-template <> struct ctype_trait<Box> {
-  static Box ret(Box t) { return t; }
-  static Box arg(Box t) { return t; }
-  static Box box(Box t) { return t; }
-  static auto add() { return sexpr<Box>::add("Box"); }
-};
-
 template <class T> void deleter(void *ptr) {
   //  std::cerr << "cc del " << reinterpret_cast<void *>(ptr) << std::endl;
   delete static_cast<T *>(ptr);
 }
 
-template <class T> struct simple_type {
+// SIMPLE
+template <class T> struct SIMPLE {
   using B = T;
   static B ret(T t) { return t; }
   static T arg(B t) { return t; }
@@ -64,9 +52,17 @@ template <class T> struct simple_type {
   }
 };
 
+template <> struct SIMPLE<void> {};
+
+template <> struct SIMPLE<Box> {
+  static Box ret(Box t) { return t; }
+  static Box arg(Box t) { return t; }
+  static Box box(Box t) { return t; }
+};
+
 #define CT(T)                                                                  \
-  template <> struct ctype_trait<T> : simple_type<T> {                         \
-    static auto add() { return sexpr<T>::add(#T); }                            \
+  template <> struct ctype<T> : SIMPLE<T> {                                    \
+    static size_t add() { return sexpr<T>::add(#T); }                          \
   };
 
 CT(bool)
@@ -89,18 +85,20 @@ CT(long double)
 
 CT(char *)
 CT(wchar_t *)
+
 CT(void *)
 CT(PyObject *)
 
+CT(void)
+CT(Box)
+
 // CONST - ignore for now
-template <class T> struct ctype_trait<const T *> : ctype_trait<T *> {
-  static Box box(const T *t) {
-    return ctype_trait<T *>::box(const_cast<T *>(t));
-  }
+template <class T> struct ctype<const T *> : ctype<T *> {
+  static Box box(const T *t) { return ctype<T *>::box(const_cast<T *>(t)); }
 };
 
 // POINTER
-template <class T> struct ctype_trait<T *> {
+template <class T> struct POINTER {
   using P = T *;
   using B = P;
 
@@ -108,14 +106,16 @@ template <class T> struct ctype_trait<T *> {
   static P arg(B b) { return b; }
 
   static Box box(P p) { return {type_id<P>(), p, nullptr}; }
+};
 
-  static auto add() { return sexpr<T *, T>::add("POINTER"); }
+template <class T> struct ctype<T *> : POINTER<T> {
+  static size_t add() { return sexpr<T *, T>::add("POINTER"); }
 };
 
 // CFUNCTYPE
-template <class Ret, class... Args> struct ctype_trait<Ret (*)(Args...)> {
+template <class Ret, class... Args> struct CFUNCTYPE {
   using F = Ret (*)(Args...);
-  using B = Ret (*)(Args...);
+  using B = F;
 
   static B ret(F f) { return f; }
   static F arg(B b) { return b; }
@@ -123,8 +123,11 @@ template <class Ret, class... Args> struct ctype_trait<Ret (*)(Args...)> {
   static Box box(F f) {
     return {type_id<F>(), reinterpret_cast<void *>(f), nullptr};
   }
+};
 
-  static auto add() {
+template <class Ret, class... Args>
+struct ctype<Ret (*)(Args...)> : CFUNCTYPE<Ret, Args...> {
+  static size_t add() {
     return sexpr<Ret (*)(Args...), Ret, Args...>::add("CFUNCTYPE");
   }
 };
@@ -137,7 +140,7 @@ struct PyResource {
 };
 
 template <class Ret, class... Args> struct Func {
-  typename ctype_trait<Ret>::B (*callback)(typename ctype_trait<Args>::B...);
+  typename ctype<Ret>::B (*callback)(typename ctype<Args>::B...);
   std::shared_ptr<PyResource> res;
 
   Func(Box &box) {
@@ -145,46 +148,38 @@ template <class Ret, class... Args> struct Func {
     res = std::make_shared<PyResource>(box.ptr, box.deleter);
   }
   Ret operator()(Args... args) {
-    return ctype_trait<Ret>::arg(callback(ctype_trait<Args>::ret(args)...));
+    return ctype<Ret>::arg(callback(ctype<Args>::ret(args)...));
   }
 };
 
 template <class... Args> struct Func<void, Args...> {
-  void (*callback)(typename ctype_trait<Args>::B...);
+  void (*callback)(typename ctype<Args>::B...);
   std::shared_ptr<PyResource> res;
 
   Func(Box &box) {
     callback = reinterpret_cast<decltype(callback)>(box.ptr);
     res = std::make_shared<PyResource>(box.ptr, box.deleter);
   }
-  void operator()(Args... args) { callback(ctype_trait<Args>::arg(args)...); }
+  void operator()(Args... args) { callback(ctype<Args>::arg(args)...); }
 };
 
 template <class F, class Ret, class... Args> struct Caller {
-  static Ret _caller(void *ptr, Args... args) {
-    auto &f = *static_cast<F *>(ptr);
-    return f(args...);
-  }
-  static typename ctype_trait<Ret>::B
-  caller(void *ptr, typename ctype_trait<Args>::B... args) {
-    return ctype_trait<Ret>::ret(_caller(ptr, ctype_trait<Args>::arg(args)...));
+  using Ret_B = typename ctype<Ret>::B;
+  static Ret_B caller(void *ptr, typename ctype<Args>::B... args) {
+    F &f = *static_cast<F *>(ptr);
+    return ctype<Ret>::ret(f(ctype<Args>::arg(args)...));
   }
 };
 
 template <class F, class... Args> struct Caller<F, void, Args...> {
-
-  static void _caller(void *ptr, Args... args) {
-    auto &f = *static_cast<F *>(ptr);
-    return f(args...);
-  }
-  static void caller(void *ptr, typename ctype_trait<Args>::B... args) {
-    _caller(ptr, ctype_trait<Args>::arg(args)...);
+  static void caller(void *ptr, typename ctype<Args>::B... args) {
+    F &f = *static_cast<F *>(ptr);
+    return f(ctype<Args>::arg(args)...);
   }
 };
 
 // CPPFUNCTYPE
-template <class F, class Ret, class... Args>
-struct ctype_trait<Ret (F::*)(Args...) const> {
+template <class F, class Ret, class... Args> struct CPPFUNCTYPE {
   using B = Box;
 
   static Box box(F f) {
@@ -201,15 +196,18 @@ struct ctype_trait<Ret (F::*)(Args...) const> {
 
   static B ret(F f) { return box(f); }
   static F arg(B b) { return unbox(b); }
+};
 
-  static auto add() {
+template <class F, class Ret, class... Args>
+struct ctype<Ret (F::*)(Args...) const> : CPPFUNCTYPE<F, Ret, Args...> {
+  static size_t add() {
     auto tid = sexpr<F, Ret, Args...>::add("CPPFUNCTYPE");
 
-    using pt = ctype_trait<decltype(Func<Ret, Args...>::callback)>;
+    using pt = ctype<decltype(Func<Ret, Args...>::callback)>;
     ts->add_callback(tid, pt::add());
 
     auto call = Caller<F, Ret, Args...>::caller;
-    using ct = ctype_trait<decltype(call)>;
+    using ct = ctype<decltype(call)>;
     ct::add();
     ts->add_caller(tid, ct::box(call));
 
@@ -217,8 +215,7 @@ struct ctype_trait<Ret (F::*)(Args...) const> {
   }
 };
 
-template <class F>
-struct ctype_trait : ctype_trait<decltype(&F::operator())> {};
+template <class F> struct ctype : ctype<decltype(&F::operator())> {};
 
 } // namespace pybindcpp
 
